# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from rapidata.api_client.models.i_validation_truth_attach_category_truth import IValidationTruthAttachCategoryTruth
from rapidata.api_client.models.i_validation_truth_bounding_box_truth import IValidationTruthBoundingBoxTruth
from rapidata.api_client.models.i_validation_truth_compare_truth import IValidationTruthCompareTruth
from rapidata.api_client.models.i_validation_truth_empty_validation_truth import IValidationTruthEmptyValidationTruth
from rapidata.api_client.models.i_validation_truth_line_truth import IValidationTruthLineTruth
from rapidata.api_client.models.i_validation_truth_locate_box_truth import IValidationTruthLocateBoxTruth
from rapidata.api_client.models.i_validation_truth_multi_compare_truth import IValidationTruthMultiCompareTruth
from rapidata.api_client.models.i_validation_truth_named_entity_truth import IValidationTruthNamedEntityTruth
from rapidata.api_client.models.i_validation_truth_polygon_truth import IValidationTruthPolygonTruth
from rapidata.api_client.models.i_validation_truth_scrub_truth import IValidationTruthScrubTruth
from rapidata.api_client.models.i_validation_truth_skip_truth import IValidationTruthSkipTruth
from rapidata.api_client.models.i_validation_truth_transcription_truth import IValidationTruthTranscriptionTruth
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

IVALIDATIONTRUTH_ONE_OF_SCHEMAS = ["IValidationTruthAttachCategoryTruth", "IValidationTruthBoundingBoxTruth", "IValidationTruthCompareTruth", "IValidationTruthEmptyValidationTruth", "IValidationTruthLineTruth", "IValidationTruthLocateBoxTruth", "IValidationTruthMultiCompareTruth", "IValidationTruthNamedEntityTruth", "IValidationTruthPolygonTruth", "IValidationTruthScrubTruth", "IValidationTruthSkipTruth", "IValidationTruthTranscriptionTruth"]

class IValidationTruth(BaseModel):
    """
    IValidationTruth
    """
    # data type: IValidationTruthTranscriptionTruth
    oneof_schema_1_validator: Optional[IValidationTruthTranscriptionTruth] = None
    # data type: IValidationTruthScrubTruth
    oneof_schema_2_validator: Optional[IValidationTruthScrubTruth] = None
    # data type: IValidationTruthPolygonTruth
    oneof_schema_3_validator: Optional[IValidationTruthPolygonTruth] = None
    # data type: IValidationTruthNamedEntityTruth
    oneof_schema_4_validator: Optional[IValidationTruthNamedEntityTruth] = None
    # data type: IValidationTruthLocateBoxTruth
    oneof_schema_5_validator: Optional[IValidationTruthLocateBoxTruth] = None
    # data type: IValidationTruthLineTruth
    oneof_schema_6_validator: Optional[IValidationTruthLineTruth] = None
    # data type: IValidationTruthEmptyValidationTruth
    oneof_schema_7_validator: Optional[IValidationTruthEmptyValidationTruth] = None
    # data type: IValidationTruthCompareTruth
    oneof_schema_8_validator: Optional[IValidationTruthCompareTruth] = None
    # data type: IValidationTruthMultiCompareTruth
    oneof_schema_9_validator: Optional[IValidationTruthMultiCompareTruth] = None
    # data type: IValidationTruthSkipTruth
    oneof_schema_10_validator: Optional[IValidationTruthSkipTruth] = None
    # data type: IValidationTruthAttachCategoryTruth
    oneof_schema_11_validator: Optional[IValidationTruthAttachCategoryTruth] = None
    # data type: IValidationTruthBoundingBoxTruth
    oneof_schema_12_validator: Optional[IValidationTruthBoundingBoxTruth] = None
    actual_instance: Optional[Union[IValidationTruthAttachCategoryTruth, IValidationTruthBoundingBoxTruth, IValidationTruthCompareTruth, IValidationTruthEmptyValidationTruth, IValidationTruthLineTruth, IValidationTruthLocateBoxTruth, IValidationTruthMultiCompareTruth, IValidationTruthNamedEntityTruth, IValidationTruthPolygonTruth, IValidationTruthScrubTruth, IValidationTruthSkipTruth, IValidationTruthTranscriptionTruth]] = None
    one_of_schemas: Set[str] = { "IValidationTruthAttachCategoryTruth", "IValidationTruthBoundingBoxTruth", "IValidationTruthCompareTruth", "IValidationTruthEmptyValidationTruth", "IValidationTruthLineTruth", "IValidationTruthLocateBoxTruth", "IValidationTruthMultiCompareTruth", "IValidationTruthNamedEntityTruth", "IValidationTruthPolygonTruth", "IValidationTruthScrubTruth", "IValidationTruthSkipTruth", "IValidationTruthTranscriptionTruth" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )


    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        instance = IValidationTruth.model_construct()
        error_messages = []
        match = 0
        # validate data type: IValidationTruthTranscriptionTruth
        if not isinstance(v, IValidationTruthTranscriptionTruth):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IValidationTruthTranscriptionTruth`")
        else:
            match += 1
        # validate data type: IValidationTruthScrubTruth
        if not isinstance(v, IValidationTruthScrubTruth):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IValidationTruthScrubTruth`")
        else:
            match += 1
        # validate data type: IValidationTruthPolygonTruth
        if not isinstance(v, IValidationTruthPolygonTruth):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IValidationTruthPolygonTruth`")
        else:
            match += 1
        # validate data type: IValidationTruthNamedEntityTruth
        if not isinstance(v, IValidationTruthNamedEntityTruth):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IValidationTruthNamedEntityTruth`")
        else:
            match += 1
        # validate data type: IValidationTruthLocateBoxTruth
        if not isinstance(v, IValidationTruthLocateBoxTruth):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IValidationTruthLocateBoxTruth`")
        else:
            match += 1
        # validate data type: IValidationTruthLineTruth
        if not isinstance(v, IValidationTruthLineTruth):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IValidationTruthLineTruth`")
        else:
            match += 1
        # validate data type: IValidationTruthEmptyValidationTruth
        if not isinstance(v, IValidationTruthEmptyValidationTruth):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IValidationTruthEmptyValidationTruth`")
        else:
            match += 1
        # validate data type: IValidationTruthCompareTruth
        if not isinstance(v, IValidationTruthCompareTruth):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IValidationTruthCompareTruth`")
        else:
            match += 1
        # validate data type: IValidationTruthMultiCompareTruth
        if not isinstance(v, IValidationTruthMultiCompareTruth):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IValidationTruthMultiCompareTruth`")
        else:
            match += 1
        # validate data type: IValidationTruthSkipTruth
        if not isinstance(v, IValidationTruthSkipTruth):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IValidationTruthSkipTruth`")
        else:
            match += 1
        # validate data type: IValidationTruthAttachCategoryTruth
        if not isinstance(v, IValidationTruthAttachCategoryTruth):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IValidationTruthAttachCategoryTruth`")
        else:
            match += 1
        # validate data type: IValidationTruthBoundingBoxTruth
        if not isinstance(v, IValidationTruthBoundingBoxTruth):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IValidationTruthBoundingBoxTruth`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in IValidationTruth with oneOf schemas: IValidationTruthAttachCategoryTruth, IValidationTruthBoundingBoxTruth, IValidationTruthCompareTruth, IValidationTruthEmptyValidationTruth, IValidationTruthLineTruth, IValidationTruthLocateBoxTruth, IValidationTruthMultiCompareTruth, IValidationTruthNamedEntityTruth, IValidationTruthPolygonTruth, IValidationTruthScrubTruth, IValidationTruthSkipTruth, IValidationTruthTranscriptionTruth. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in IValidationTruth with oneOf schemas: IValidationTruthAttachCategoryTruth, IValidationTruthBoundingBoxTruth, IValidationTruthCompareTruth, IValidationTruthEmptyValidationTruth, IValidationTruthLineTruth, IValidationTruthLocateBoxTruth, IValidationTruthMultiCompareTruth, IValidationTruthNamedEntityTruth, IValidationTruthPolygonTruth, IValidationTruthScrubTruth, IValidationTruthSkipTruth, IValidationTruthTranscriptionTruth. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        match = 0

        # deserialize data into IValidationTruthTranscriptionTruth
        try:
            instance.actual_instance = IValidationTruthTranscriptionTruth.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IValidationTruthScrubTruth
        try:
            instance.actual_instance = IValidationTruthScrubTruth.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IValidationTruthPolygonTruth
        try:
            instance.actual_instance = IValidationTruthPolygonTruth.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IValidationTruthNamedEntityTruth
        try:
            instance.actual_instance = IValidationTruthNamedEntityTruth.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IValidationTruthLocateBoxTruth
        try:
            instance.actual_instance = IValidationTruthLocateBoxTruth.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IValidationTruthLineTruth
        try:
            instance.actual_instance = IValidationTruthLineTruth.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IValidationTruthEmptyValidationTruth
        try:
            instance.actual_instance = IValidationTruthEmptyValidationTruth.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IValidationTruthCompareTruth
        try:
            instance.actual_instance = IValidationTruthCompareTruth.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IValidationTruthMultiCompareTruth
        try:
            instance.actual_instance = IValidationTruthMultiCompareTruth.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IValidationTruthSkipTruth
        try:
            instance.actual_instance = IValidationTruthSkipTruth.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IValidationTruthAttachCategoryTruth
        try:
            instance.actual_instance = IValidationTruthAttachCategoryTruth.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IValidationTruthBoundingBoxTruth
        try:
            instance.actual_instance = IValidationTruthBoundingBoxTruth.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into IValidationTruth with oneOf schemas: IValidationTruthAttachCategoryTruth, IValidationTruthBoundingBoxTruth, IValidationTruthCompareTruth, IValidationTruthEmptyValidationTruth, IValidationTruthLineTruth, IValidationTruthLocateBoxTruth, IValidationTruthMultiCompareTruth, IValidationTruthNamedEntityTruth, IValidationTruthPolygonTruth, IValidationTruthScrubTruth, IValidationTruthSkipTruth, IValidationTruthTranscriptionTruth. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into IValidationTruth with oneOf schemas: IValidationTruthAttachCategoryTruth, IValidationTruthBoundingBoxTruth, IValidationTruthCompareTruth, IValidationTruthEmptyValidationTruth, IValidationTruthLineTruth, IValidationTruthLocateBoxTruth, IValidationTruthMultiCompareTruth, IValidationTruthNamedEntityTruth, IValidationTruthPolygonTruth, IValidationTruthScrubTruth, IValidationTruthSkipTruth, IValidationTruthTranscriptionTruth. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], IValidationTruthAttachCategoryTruth, IValidationTruthBoundingBoxTruth, IValidationTruthCompareTruth, IValidationTruthEmptyValidationTruth, IValidationTruthLineTruth, IValidationTruthLocateBoxTruth, IValidationTruthMultiCompareTruth, IValidationTruthNamedEntityTruth, IValidationTruthPolygonTruth, IValidationTruthScrubTruth, IValidationTruthSkipTruth, IValidationTruthTranscriptionTruth]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())





from rapidata.rapidata_client.selection._base_selection import RapidataSelection


class StaticSelection(RapidataSelection):
    """StaticSelection Class

    Given a list of RapidIds, theses specific rapids will be shown in order for every session.

    Args:
        rapid_ids (list[str]): List of rapid ids to show.
    """

    def __init__(self, rapid_ids: list[str]):
        self.rapid_ids = rapid_ids

    def _to_model(self):
        from rapidata.api_client.models.i_selection import ISelection
        from rapidata.api_client.models.i_selection_static_selection import (
            ISelectionStaticSelection,
        )

        return ISelection(
            actual_instance=ISelectionStaticSelection(
                _t="StaticSelection",
                rapidIds=self.rapid_ids,
            )
        )

    def __str__(self) -> str:
        return f"StaticSelection(rapid_ids={self.rapid_ids})"

    def __repr__(self) -> str:
        return self.__str__()



# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from rapidata.api_client.models.i_rapid_result_attach_category_result import IRapidResultAttachCategoryResult
from rapidata.api_client.models.i_rapid_result_bounding_box_result import IRapidResultBoundingBoxResult
from rapidata.api_client.models.i_rapid_result_compare_result import IRapidResultCompareResult
from rapidata.api_client.models.i_rapid_result_free_text_result import IRapidResultFreeTextResult
from rapidata.api_client.models.i_rapid_result_line_result import IRapidResultLineResult
from rapidata.api_client.models.i_rapid_result_locate_result import IRapidResultLocateResult
from rapidata.api_client.models.i_rapid_result_named_entity_result import IRapidResultNamedEntityResult
from rapidata.api_client.models.i_rapid_result_polygon_result import IRapidResultPolygonResult
from rapidata.api_client.models.i_rapid_result_scrub_result import IRapidResultScrubResult
from rapidata.api_client.models.i_rapid_result_skip_result import IRapidResultSkipResult
from rapidata.api_client.models.i_rapid_result_transcription_result import IRapidResultTranscriptionResult
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

IRAPIDRESULT_ONE_OF_SCHEMAS = ["IRapidResultAttachCategoryResult", "IRapidResultBoundingBoxResult", "IRapidResultCompareResult", "IRapidResultFreeTextResult", "IRapidResultLineResult", "IRapidResultLocateResult", "IRapidResultNamedEntityResult", "IRapidResultPolygonResult", "IRapidResultScrubResult", "IRapidResultSkipResult", "IRapidResultTranscriptionResult"]

class IRapidResult(BaseModel):
    """
    IRapidResult
    """
    # data type: IRapidResultTranscriptionResult
    oneof_schema_1_validator: Optional[IRapidResultTranscriptionResult] = None
    # data type: IRapidResultScrubResult
    oneof_schema_2_validator: Optional[IRapidResultScrubResult] = None
    # data type: IRapidResultPolygonResult
    oneof_schema_3_validator: Optional[IRapidResultPolygonResult] = None
    # data type: IRapidResultNamedEntityResult
    oneof_schema_4_validator: Optional[IRapidResultNamedEntityResult] = None
    # data type: IRapidResultLocateResult
    oneof_schema_5_validator: Optional[IRapidResultLocateResult] = None
    # data type: IRapidResultLineResult
    oneof_schema_6_validator: Optional[IRapidResultLineResult] = None
    # data type: IRapidResultFreeTextResult
    oneof_schema_7_validator: Optional[IRapidResultFreeTextResult] = None
    # data type: IRapidResultCompareResult
    oneof_schema_8_validator: Optional[IRapidResultCompareResult] = None
    # data type: IRapidResultSkipResult
    oneof_schema_9_validator: Optional[IRapidResultSkipResult] = None
    # data type: IRapidResultAttachCategoryResult
    oneof_schema_10_validator: Optional[IRapidResultAttachCategoryResult] = None
    # data type: IRapidResultBoundingBoxResult
    oneof_schema_11_validator: Optional[IRapidResultBoundingBoxResult] = None
    actual_instance: Optional[Union[IRapidResultAttachCategoryResult, IRapidResultBoundingBoxResult, IRapidResultCompareResult, IRapidResultFreeTextResult, IRapidResultLineResult, IRapidResultLocateResult, IRapidResultNamedEntityResult, IRapidResultPolygonResult, IRapidResultScrubResult, IRapidResultSkipResult, IRapidResultTranscriptionResult]] = None
    one_of_schemas: Set[str] = { "IRapidResultAttachCategoryResult", "IRapidResultBoundingBoxResult", "IRapidResultCompareResult", "IRapidResultFreeTextResult", "IRapidResultLineResult", "IRapidResultLocateResult", "IRapidResultNamedEntityResult", "IRapidResultPolygonResult", "IRapidResultScrubResult", "IRapidResultSkipResult", "IRapidResultTranscriptionResult" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )


    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        instance = IRapidResult.model_construct()
        error_messages = []
        match = 0
        # validate data type: IRapidResultTranscriptionResult
        if not isinstance(v, IRapidResultTranscriptionResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidResultTranscriptionResult`")
        else:
            match += 1
        # validate data type: IRapidResultScrubResult
        if not isinstance(v, IRapidResultScrubResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidResultScrubResult`")
        else:
            match += 1
        # validate data type: IRapidResultPolygonResult
        if not isinstance(v, IRapidResultPolygonResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidResultPolygonResult`")
        else:
            match += 1
        # validate data type: IRapidResultNamedEntityResult
        if not isinstance(v, IRapidResultNamedEntityResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidResultNamedEntityResult`")
        else:
            match += 1
        # validate data type: IRapidResultLocateResult
        if not isinstance(v, IRapidResultLocateResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidResultLocateResult`")
        else:
            match += 1
        # validate data type: IRapidResultLineResult
        if not isinstance(v, IRapidResultLineResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidResultLineResult`")
        else:
            match += 1
        # validate data type: IRapidResultFreeTextResult
        if not isinstance(v, IRapidResultFreeTextResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidResultFreeTextResult`")
        else:
            match += 1
        # validate data type: IRapidResultCompareResult
        if not isinstance(v, IRapidResultCompareResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidResultCompareResult`")
        else:
            match += 1
        # validate data type: IRapidResultSkipResult
        if not isinstance(v, IRapidResultSkipResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidResultSkipResult`")
        else:
            match += 1
        # validate data type: IRapidResultAttachCategoryResult
        if not isinstance(v, IRapidResultAttachCategoryResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidResultAttachCategoryResult`")
        else:
            match += 1
        # validate data type: IRapidResultBoundingBoxResult
        if not isinstance(v, IRapidResultBoundingBoxResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidResultBoundingBoxResult`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in IRapidResult with oneOf schemas: IRapidResultAttachCategoryResult, IRapidResultBoundingBoxResult, IRapidResultCompareResult, IRapidResultFreeTextResult, IRapidResultLineResult, IRapidResultLocateResult, IRapidResultNamedEntityResult, IRapidResultPolygonResult, IRapidResultScrubResult, IRapidResultSkipResult, IRapidResultTranscriptionResult. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in IRapidResult with oneOf schemas: IRapidResultAttachCategoryResult, IRapidResultBoundingBoxResult, IRapidResultCompareResult, IRapidResultFreeTextResult, IRapidResultLineResult, IRapidResultLocateResult, IRapidResultNamedEntityResult, IRapidResultPolygonResult, IRapidResultScrubResult, IRapidResultSkipResult, IRapidResultTranscriptionResult. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        match = 0

        # deserialize data into IRapidResultTranscriptionResult
        try:
            instance.actual_instance = IRapidResultTranscriptionResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidResultScrubResult
        try:
            instance.actual_instance = IRapidResultScrubResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidResultPolygonResult
        try:
            instance.actual_instance = IRapidResultPolygonResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidResultNamedEntityResult
        try:
            instance.actual_instance = IRapidResultNamedEntityResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidResultLocateResult
        try:
            instance.actual_instance = IRapidResultLocateResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidResultLineResult
        try:
            instance.actual_instance = IRapidResultLineResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidResultFreeTextResult
        try:
            instance.actual_instance = IRapidResultFreeTextResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidResultCompareResult
        try:
            instance.actual_instance = IRapidResultCompareResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidResultSkipResult
        try:
            instance.actual_instance = IRapidResultSkipResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidResultAttachCategoryResult
        try:
            instance.actual_instance = IRapidResultAttachCategoryResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidResultBoundingBoxResult
        try:
            instance.actual_instance = IRapidResultBoundingBoxResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into IRapidResult with oneOf schemas: IRapidResultAttachCategoryResult, IRapidResultBoundingBoxResult, IRapidResultCompareResult, IRapidResultFreeTextResult, IRapidResultLineResult, IRapidResultLocateResult, IRapidResultNamedEntityResult, IRapidResultPolygonResult, IRapidResultScrubResult, IRapidResultSkipResult, IRapidResultTranscriptionResult. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into IRapidResult with oneOf schemas: IRapidResultAttachCategoryResult, IRapidResultBoundingBoxResult, IRapidResultCompareResult, IRapidResultFreeTextResult, IRapidResultLineResult, IRapidResultLocateResult, IRapidResultNamedEntityResult, IRapidResultPolygonResult, IRapidResultScrubResult, IRapidResultSkipResult, IRapidResultTranscriptionResult. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], IRapidResultAttachCategoryResult, IRapidResultBoundingBoxResult, IRapidResultCompareResult, IRapidResultFreeTextResult, IRapidResultLineResult, IRapidResultLocateResult, IRapidResultNamedEntityResult, IRapidResultPolygonResult, IRapidResultScrubResult, IRapidResultSkipResult, IRapidResultTranscriptionResult]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())





from __future__ import annotations

from typing import TYPE_CHECKING

from rapidata.rapidata_client.datapoints._datapoint import Datapoint
from rapidata.service.openapi_service import OpenAPIService
from rapidata.rapidata_client.datapoints._asset_uploader import AssetUploader

if TYPE_CHECKING:
    from rapidata.api_client.models.create_datapoint_result import CreateDatapointResult


class DatapointUploader:
    def __init__(self, openapi_service: OpenAPIService):
        self.openapi_service = openapi_service
        self.asset_uploader = AssetUploader(openapi_service)

    def upload_datapoint(
        self, datapoint: Datapoint, dataset_id: str, index: int
    ) -> CreateDatapointResult:
        from rapidata.api_client.models.create_datapoint_model import (
            CreateDatapointModel,
        )
        from rapidata.api_client.models.i_asset_input import IAssetInput

        uploaded_asset: IAssetInput = (
            self.asset_uploader.get_uploaded_asset_input(datapoint.asset)
            if datapoint.data_type == "media"
            else self.asset_uploader.get_uploaded_text_input(datapoint.asset)
        )
        return self.openapi_service.dataset_api.dataset_dataset_id_datapoint_post(
            dataset_id=dataset_id,
            create_datapoint_model=CreateDatapointModel(
                asset=uploaded_asset,
                context=datapoint.context,
                contextAsset=(
                    self.asset_uploader.get_uploaded_asset_input(
                        datapoint.media_context
                    )
                    if datapoint.media_context
                    else None
                ),
                transcription=datapoint.sentence,
                sortIndex=index,
                group=datapoint.group,
            ),
        )



# coding: utf-8

"""
    Rapidata.Dataset

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from rapidata.api_client.models.create_datapoint_from_files_model_metadata_inner import CreateDatapointFromFilesModelMetadataInner
from typing import Optional, Set
from typing_extensions import Self

class CreateDatapointFromUrlsModel(BaseModel):
    """
    The body request for creating a datapoint from urls.
    """ # noqa: E501
    urls: List[StrictStr] = Field(description="The urls to fetch the assets from. The urls must be publicly accessible. A HEAD request will be made to each url to check if it is accessible.")
    metadata: Optional[List[CreateDatapointFromFilesModelMetadataInner]] = Field(default=None, description="Additional metadata to attach to the datapoint. Most commonly used to add a prompt to the datapoint using the Rapidata.Shared.Assets.Abstraction.Models.Metadata.Input.PromptMetadataInput.")
    sort_index: Optional[StrictInt] = Field(default=None, description="The index will be used to keep the datapoints in order. Useful if upload is parallelized", alias="sortIndex")
    __properties: ClassVar[List[str]] = ["urls", "metadata", "sortIndex"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateDatapointFromUrlsModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in metadata (list)
        _items = []
        if self.metadata:
            for _item_metadata in self.metadata:
                if _item_metadata:
                    _items.append(_item_metadata.to_dict())
            _dict['metadata'] = _items
        # set to None if metadata (nullable) is None
        # and model_fields_set contains the field
        if self.metadata is None and "metadata" in self.model_fields_set:
            _dict['metadata'] = None

        # set to None if sort_index (nullable) is None
        # and model_fields_set contains the field
        if self.sort_index is None and "sort_index" in self.model_fields_set:
            _dict['sortIndex'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateDatapointFromUrlsModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "urls": obj.get("urls"),
            "metadata": [CreateDatapointFromFilesModelMetadataInner.from_dict(_item) for _item in obj["metadata"]] if obj.get("metadata") is not None else None,
            "sortIndex": obj.get("sortIndex")
        })
        return _obj





from pathlib import Path
from typing import Callable
from pydantic import BaseModel, Field, field_validator
import shutil
from rapidata.rapidata_client.config import logger

# Type alias for config update handlers
UploadConfigUpdateHandler = Callable[["UploadConfig"], None]

# Global list to store registered handlers
_upload_config_handlers: list[UploadConfigUpdateHandler] = []


def register_upload_config_handler(handler: UploadConfigUpdateHandler) -> None:
    """Register a handler to be called when the upload configuration updates."""
    _upload_config_handlers.append(handler)


def unregister_upload_config_handler(handler: UploadConfigUpdateHandler) -> None:
    """Unregister a previously registered handler."""
    if handler in _upload_config_handlers:
        _upload_config_handlers.remove(handler)


class UploadConfig(BaseModel):
    """
    Holds the configuration for the upload process.

    Attributes:
        maxWorkers (int): The maximum number of worker threads for processing media paths. Defaults to 25.
        maxRetries (int): The maximum number of retries for failed uploads. Defaults to 3.
    """

    maxWorkers: int = Field(default=25)
    maxRetries: int = Field(default=3)
    cacheUploads: bool = Field(default=True)
    cacheTimeout: float = Field(default=0.1)
    cacheLocation: Path = Field(
        default=Path.home() / ".cache" / "rapidata" / "upload_cache"
    )

    @field_validator("maxWorkers")
    @classmethod
    def validate_max_workers(cls, v: int) -> int:
        if v > 200:
            logger.warning(
                f"maxWorkers is set to {v}, which is above the recommended limit of 200 and default optimum of 25. "
                "This may lead to suboptimal performance due to connection pool constraints."
            )
        return v

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._notify_handlers()
        self._migrate_cache()

    def __setattr__(self, name: str, value) -> None:
        super().__setattr__(name, value)
        self._notify_handlers()

    def _notify_handlers(self) -> None:
        """Notify all registered handlers that the configuration has updated."""
        for handler in _upload_config_handlers:
            try:
                handler(self)
            except Exception as e:
                logger.warning(f"Warning: UploadConfig handler failed: {e}")

    def _migrate_cache(self) -> None:
        """Migrate the cache from the old location to the new location."""
        old_cache = Path.home() / ".rapidata" / "upload_cache"
        new_cache = self.cacheLocation
        if old_cache.exists() and not new_cache.exists():
            logger.info(f"Migrating cache from {old_cache} to {self.cacheLocation}")
            self.cacheLocation.parent.mkdir(parents=True, exist_ok=True)
            shutil.move(str(old_cache), str(self.cacheLocation))

            # Clean up old directory if empty
            if old_cache.parent.exists() and not any(old_cache.parent.iterdir()):
                old_cache.parent.rmdir()



from rapidata.api_client.models.i_order_workflow_model import IOrderWorkflowModel
from rapidata.api_client.models.i_order_workflow_model_simple_workflow_model import (
    IOrderWorkflowModelSimpleWorkflowModel,
)
from rapidata.api_client.models.i_rapid_blueprint import IRapidBlueprint
from rapidata.api_client.models.i_rapid_blueprint_transcription_rapid_blueprint import (
    IRapidBlueprintTranscriptionRapidBlueprint,
)
from rapidata.rapidata_client.workflow._base_workflow import Workflow
from rapidata.api_client.models.i_rapid_payload_transcription_payload import (
    IRapidPayloadTranscriptionPayload,
)
from rapidata.api_client.models.transcription_word import TranscriptionWord
from rapidata.rapidata_client.datapoints._datapoint import Datapoint
from rapidata.api_client.models.rapid_modality import RapidModality
from rapidata.api_client.models.i_rapid_payload import IRapidPayload


class SelectWordsWorkflow(Workflow):
    """
    A workflow for select words tasks.

    This class represents a select words workflow
    where datapoints have a sentence attached to them where words can be selected.

    Attributes:
        _instruction (str): The instruction for the select words task.

    Args:
        instruction (str): The instruction to be provided for the select words task.
    """

    modality = RapidModality.TRANSCRIPTION

    def __init__(self, instruction: str):
        super().__init__(type="SimpleWorkflowConfig")
        self._instruction = instruction

    def _get_instruction(self) -> str:
        return self._instruction

    def _to_model(self) -> IOrderWorkflowModel:
        blueprint = IRapidBlueprintTranscriptionRapidBlueprint(
            _t="TranscriptionBlueprint", title=self._instruction
        )

        return IOrderWorkflowModel(
            actual_instance=IOrderWorkflowModelSimpleWorkflowModel(
                _t="SimpleWorkflow",
                blueprint=IRapidBlueprint(actual_instance=blueprint),
            )
        )

    def _to_payload(self, datapoint: Datapoint) -> IRapidPayload:
        assert (
            datapoint.sentence is not None
        ), "SelectWordsWorkflow requires a sentence datapoint"

        return IRapidPayload(
            actual_instance=IRapidPayloadTranscriptionPayload(
                _t="TranscriptionPayload",
                title=self._instruction,
                transcription=[
                    TranscriptionWord(word=word, wordIndex=i)
                    for i, word in enumerate(datapoint.sentence.split())
                ],
            )
        )

    def __str__(self) -> str:
        return f"SelectWordsWorkflow(instruction='{self._instruction}')"

    def __repr__(self) -> str:
        return f"SelectWordsWorkflow(instruction={self._instruction!r})"



from rapidata.rapidata_client.settings._rapidata_setting import RapidataSetting
from rapidata.rapidata_client.config import managed_print


class AlertOnFastResponse(RapidataSetting):
    """
    Gives an alert as a pop up on the UI when the response time is less than the milliseconds.

    Args:
        threshold (int): if the user responds in less than this time, an alert will be shown.
    """

    def __init__(self, threshold: int):
        if not isinstance(threshold, int):
            raise ValueError("The alert must be an integer.")
        if threshold < 10:
            managed_print(
                f"Warning: Are you sure you want to set the threshold so low ({threshold} milliseconds)?"
            )
        if threshold > 25000:
            raise ValueError("The alert must be less than 25000 milliseconds.")
        if threshold < 0:
            raise ValueError("The alert must be greater than or equal to 0.")

        super().__init__(key="alert_on_fast_response", value=threshold)



from __future__ import annotations

import re
import os
from typing import TYPE_CHECKING, cast

from rapidata.rapidata_client.config.upload_config import register_upload_config_handler
from rapidata.rapidata_client.config.upload_config import UploadConfig
from rapidata.service.openapi_service import OpenAPIService
from rapidata.rapidata_client.config import logger
from rapidata.rapidata_client.config import tracer
from rapidata.rapidata_client.config import rapidata_config
from diskcache import FanoutCache

if TYPE_CHECKING:
    from rapidata.api_client.models.i_asset_input import IAssetInput


class AssetUploader:
    _shared_upload_cache: FanoutCache = FanoutCache(
        rapidata_config.upload.cacheLocation,
        shards=rapidata_config.upload.maxWorkers,
        timeout=rapidata_config.upload.cacheTimeout,
    )

    def __init__(self, openapi_service: OpenAPIService):
        self.openapi_service = openapi_service
        # Register the handler for config update changes
        register_upload_config_handler(self._handle_config_update)

    @classmethod
    def _handle_config_update(cls, config: UploadConfig):
        """
        Handle updates to the upload config (e.g., cache location, size, workers, timeout).
        Re-instantiates the shared cache based on the new configuration.
        """
        logger.debug("Updating AssetUploader shared upload cache with new config")
        try:
            # Dispose of the old cache (closes open resources)
            old_cache = getattr(cls, "_shared_upload_cache", None)
            if old_cache:
                try:
                    old_cache.close()
                except Exception:
                    pass  # ignore close errors

            # Re-initialize with the updated config
            cls._shared_upload_cache = FanoutCache(
                config.cacheLocation,
                shards=config.maxWorkers,
                timeout=config.cacheTimeout,
            )
            logger.info(
                "AssetUploader shared upload cache updated: location=%s, shards=%s, timeout=%s",
                config.cacheLocation,
                config.maxWorkers,
                config.cacheTimeout,
            )

        except Exception as e:
            logger.warning(f"Failed to update AssetUploader shared upload cache: {e}")

    def _get_cache_key(self, asset: str) -> str:
        """Generate cache key for an asset, including environment."""
        env = self.openapi_service.environment
        if not os.path.exists(asset):
            raise FileNotFoundError(f"File not found: {asset}")

        stat = os.stat(asset)
        return f"{env}@{asset}:{stat.st_size}:{stat.st_mtime_ns}"

    def upload_asset(self, asset: str) -> str:
        with tracer.start_as_current_span("AssetUploader.upload_asset"):
            logger.debug("Uploading asset: %s", asset)
            assert isinstance(asset, str), "Asset must be a string"
            if re.match(r"^https?://", asset):
                response = self.openapi_service.asset_api.asset_url_post(
                    url=asset,
                )
                return response.file_name

            asset_key = self._get_cache_key(asset)
            cached_value = self._shared_upload_cache.get(asset_key)
            if cached_value is not None:
                logger.debug("Asset found in cache")
                return cast(str, cached_value)  # Type hint for the linter

            response = self.openapi_service.asset_api.asset_file_post(
                file=asset,
            )
            if rapidata_config.upload.cacheUploads:
                self._shared_upload_cache[asset_key] = response.file_name
                logger.debug("Asset added to cache")
            logger.info("Asset uploaded: %s", response.file_name)
            return response.file_name

    def get_uploaded_text_input(self, assets: list[str] | str) -> IAssetInput:
        from rapidata.api_client.models.i_asset_input import IAssetInput
        from rapidata.api_client.models.i_asset_input_text_asset_input import (
            IAssetInputTextAssetInput,
        )
        from rapidata.api_client.models.i_asset_input_multi_asset_input import (
            IAssetInputMultiAssetInput,
        )

        if isinstance(assets, list):
            return IAssetInput(
                actual_instance=IAssetInputMultiAssetInput(
                    _t="MultiAssetInput",
                    assets=[
                        IAssetInput(
                            actual_instance=IAssetInputTextAssetInput(
                                _t="TextAssetInput", text=asset
                            )
                        )
                        for asset in assets
                    ],
                )
            )
        else:
            return IAssetInput(
                actual_instance=IAssetInputTextAssetInput(
                    _t="TextAssetInput", text=assets
                )
            )

    def get_uploaded_asset_input(self, assets: list[str] | str) -> IAssetInput:
        from rapidata.api_client.models.i_asset_input import IAssetInput
        from rapidata.api_client.models.i_asset_input_existing_asset_input import (
            IAssetInputExistingAssetInput,
        )
        from rapidata.api_client.models.i_asset_input_multi_asset_input import (
            IAssetInputMultiAssetInput,
        )

        if isinstance(assets, list):
            return IAssetInput(
                actual_instance=IAssetInputMultiAssetInput(
                    _t="MultiAssetInput",
                    assets=[
                        IAssetInput(
                            actual_instance=IAssetInputExistingAssetInput(
                                _t="ExistingAssetInput",
                                name=self.upload_asset(asset),
                            )
                        )
                        for asset in assets
                    ],
                )
            )
        else:
            return IAssetInput(
                actual_instance=IAssetInputExistingAssetInput(
                    _t="ExistingAssetInput",
                    name=self.upload_asset(assets),
                )
            )

    def clear_cache(self):
        self._shared_upload_cache.clear()
        logger.info("Upload cache cleared")

    def __str__(self) -> str:
        return f"AssetUploader(openapi_service={self.openapi_service})"

    def __repr__(self) -> str:
        return f"AssetUploader(openapi_service={self.openapi_service})"



# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from rapidata.api_client.models.i_rapid_blueprint_attach_category_rapid_blueprint import IRapidBlueprintAttachCategoryRapidBlueprint
from rapidata.api_client.models.i_rapid_blueprint_bounding_box_rapid_blueprint import IRapidBlueprintBoundingBoxRapidBlueprint
from rapidata.api_client.models.i_rapid_blueprint_compare_rapid_blueprint import IRapidBlueprintCompareRapidBlueprint
from rapidata.api_client.models.i_rapid_blueprint_free_text_rapid_blueprint import IRapidBlueprintFreeTextRapidBlueprint
from rapidata.api_client.models.i_rapid_blueprint_line_rapid_blueprint import IRapidBlueprintLineRapidBlueprint
from rapidata.api_client.models.i_rapid_blueprint_locate_rapid_blueprint import IRapidBlueprintLocateRapidBlueprint
from rapidata.api_client.models.i_rapid_blueprint_named_entity_rapid_blueprint import IRapidBlueprintNamedEntityRapidBlueprint
from rapidata.api_client.models.i_rapid_blueprint_polygon_rapid_blueprint import IRapidBlueprintPolygonRapidBlueprint
from rapidata.api_client.models.i_rapid_blueprint_scrub_rapid_blueprint import IRapidBlueprintScrubRapidBlueprint
from rapidata.api_client.models.i_rapid_blueprint_transcription_rapid_blueprint import IRapidBlueprintTranscriptionRapidBlueprint
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

IRAPIDBLUEPRINT_ONE_OF_SCHEMAS = ["IRapidBlueprintAttachCategoryRapidBlueprint", "IRapidBlueprintBoundingBoxRapidBlueprint", "IRapidBlueprintCompareRapidBlueprint", "IRapidBlueprintFreeTextRapidBlueprint", "IRapidBlueprintLineRapidBlueprint", "IRapidBlueprintLocateRapidBlueprint", "IRapidBlueprintNamedEntityRapidBlueprint", "IRapidBlueprintPolygonRapidBlueprint", "IRapidBlueprintScrubRapidBlueprint", "IRapidBlueprintTranscriptionRapidBlueprint"]

class IRapidBlueprint(BaseModel):
    """
    IRapidBlueprint
    """
    # data type: IRapidBlueprintTranscriptionRapidBlueprint
    oneof_schema_1_validator: Optional[IRapidBlueprintTranscriptionRapidBlueprint] = None
    # data type: IRapidBlueprintScrubRapidBlueprint
    oneof_schema_2_validator: Optional[IRapidBlueprintScrubRapidBlueprint] = None
    # data type: IRapidBlueprintPolygonRapidBlueprint
    oneof_schema_3_validator: Optional[IRapidBlueprintPolygonRapidBlueprint] = None
    # data type: IRapidBlueprintNamedEntityRapidBlueprint
    oneof_schema_4_validator: Optional[IRapidBlueprintNamedEntityRapidBlueprint] = None
    # data type: IRapidBlueprintLocateRapidBlueprint
    oneof_schema_5_validator: Optional[IRapidBlueprintLocateRapidBlueprint] = None
    # data type: IRapidBlueprintLineRapidBlueprint
    oneof_schema_6_validator: Optional[IRapidBlueprintLineRapidBlueprint] = None
    # data type: IRapidBlueprintFreeTextRapidBlueprint
    oneof_schema_7_validator: Optional[IRapidBlueprintFreeTextRapidBlueprint] = None
    # data type: IRapidBlueprintCompareRapidBlueprint
    oneof_schema_8_validator: Optional[IRapidBlueprintCompareRapidBlueprint] = None
    # data type: IRapidBlueprintAttachCategoryRapidBlueprint
    oneof_schema_9_validator: Optional[IRapidBlueprintAttachCategoryRapidBlueprint] = None
    # data type: IRapidBlueprintBoundingBoxRapidBlueprint
    oneof_schema_10_validator: Optional[IRapidBlueprintBoundingBoxRapidBlueprint] = None
    actual_instance: Optional[Union[IRapidBlueprintAttachCategoryRapidBlueprint, IRapidBlueprintBoundingBoxRapidBlueprint, IRapidBlueprintCompareRapidBlueprint, IRapidBlueprintFreeTextRapidBlueprint, IRapidBlueprintLineRapidBlueprint, IRapidBlueprintLocateRapidBlueprint, IRapidBlueprintNamedEntityRapidBlueprint, IRapidBlueprintPolygonRapidBlueprint, IRapidBlueprintScrubRapidBlueprint, IRapidBlueprintTranscriptionRapidBlueprint]] = None
    one_of_schemas: Set[str] = { "IRapidBlueprintAttachCategoryRapidBlueprint", "IRapidBlueprintBoundingBoxRapidBlueprint", "IRapidBlueprintCompareRapidBlueprint", "IRapidBlueprintFreeTextRapidBlueprint", "IRapidBlueprintLineRapidBlueprint", "IRapidBlueprintLocateRapidBlueprint", "IRapidBlueprintNamedEntityRapidBlueprint", "IRapidBlueprintPolygonRapidBlueprint", "IRapidBlueprintScrubRapidBlueprint", "IRapidBlueprintTranscriptionRapidBlueprint" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )


    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        instance = IRapidBlueprint.model_construct()
        error_messages = []
        match = 0
        # validate data type: IRapidBlueprintTranscriptionRapidBlueprint
        if not isinstance(v, IRapidBlueprintTranscriptionRapidBlueprint):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidBlueprintTranscriptionRapidBlueprint`")
        else:
            match += 1
        # validate data type: IRapidBlueprintScrubRapidBlueprint
        if not isinstance(v, IRapidBlueprintScrubRapidBlueprint):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidBlueprintScrubRapidBlueprint`")
        else:
            match += 1
        # validate data type: IRapidBlueprintPolygonRapidBlueprint
        if not isinstance(v, IRapidBlueprintPolygonRapidBlueprint):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidBlueprintPolygonRapidBlueprint`")
        else:
            match += 1
        # validate data type: IRapidBlueprintNamedEntityRapidBlueprint
        if not isinstance(v, IRapidBlueprintNamedEntityRapidBlueprint):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidBlueprintNamedEntityRapidBlueprint`")
        else:
            match += 1
        # validate data type: IRapidBlueprintLocateRapidBlueprint
        if not isinstance(v, IRapidBlueprintLocateRapidBlueprint):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidBlueprintLocateRapidBlueprint`")
        else:
            match += 1
        # validate data type: IRapidBlueprintLineRapidBlueprint
        if not isinstance(v, IRapidBlueprintLineRapidBlueprint):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidBlueprintLineRapidBlueprint`")
        else:
            match += 1
        # validate data type: IRapidBlueprintFreeTextRapidBlueprint
        if not isinstance(v, IRapidBlueprintFreeTextRapidBlueprint):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidBlueprintFreeTextRapidBlueprint`")
        else:
            match += 1
        # validate data type: IRapidBlueprintCompareRapidBlueprint
        if not isinstance(v, IRapidBlueprintCompareRapidBlueprint):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidBlueprintCompareRapidBlueprint`")
        else:
            match += 1
        # validate data type: IRapidBlueprintAttachCategoryRapidBlueprint
        if not isinstance(v, IRapidBlueprintAttachCategoryRapidBlueprint):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidBlueprintAttachCategoryRapidBlueprint`")
        else:
            match += 1
        # validate data type: IRapidBlueprintBoundingBoxRapidBlueprint
        if not isinstance(v, IRapidBlueprintBoundingBoxRapidBlueprint):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IRapidBlueprintBoundingBoxRapidBlueprint`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in IRapidBlueprint with oneOf schemas: IRapidBlueprintAttachCategoryRapidBlueprint, IRapidBlueprintBoundingBoxRapidBlueprint, IRapidBlueprintCompareRapidBlueprint, IRapidBlueprintFreeTextRapidBlueprint, IRapidBlueprintLineRapidBlueprint, IRapidBlueprintLocateRapidBlueprint, IRapidBlueprintNamedEntityRapidBlueprint, IRapidBlueprintPolygonRapidBlueprint, IRapidBlueprintScrubRapidBlueprint, IRapidBlueprintTranscriptionRapidBlueprint. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in IRapidBlueprint with oneOf schemas: IRapidBlueprintAttachCategoryRapidBlueprint, IRapidBlueprintBoundingBoxRapidBlueprint, IRapidBlueprintCompareRapidBlueprint, IRapidBlueprintFreeTextRapidBlueprint, IRapidBlueprintLineRapidBlueprint, IRapidBlueprintLocateRapidBlueprint, IRapidBlueprintNamedEntityRapidBlueprint, IRapidBlueprintPolygonRapidBlueprint, IRapidBlueprintScrubRapidBlueprint, IRapidBlueprintTranscriptionRapidBlueprint. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        match = 0

        # deserialize data into IRapidBlueprintTranscriptionRapidBlueprint
        try:
            instance.actual_instance = IRapidBlueprintTranscriptionRapidBlueprint.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidBlueprintScrubRapidBlueprint
        try:
            instance.actual_instance = IRapidBlueprintScrubRapidBlueprint.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidBlueprintPolygonRapidBlueprint
        try:
            instance.actual_instance = IRapidBlueprintPolygonRapidBlueprint.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidBlueprintNamedEntityRapidBlueprint
        try:
            instance.actual_instance = IRapidBlueprintNamedEntityRapidBlueprint.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidBlueprintLocateRapidBlueprint
        try:
            instance.actual_instance = IRapidBlueprintLocateRapidBlueprint.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidBlueprintLineRapidBlueprint
        try:
            instance.actual_instance = IRapidBlueprintLineRapidBlueprint.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidBlueprintFreeTextRapidBlueprint
        try:
            instance.actual_instance = IRapidBlueprintFreeTextRapidBlueprint.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidBlueprintCompareRapidBlueprint
        try:
            instance.actual_instance = IRapidBlueprintCompareRapidBlueprint.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidBlueprintAttachCategoryRapidBlueprint
        try:
            instance.actual_instance = IRapidBlueprintAttachCategoryRapidBlueprint.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IRapidBlueprintBoundingBoxRapidBlueprint
        try:
            instance.actual_instance = IRapidBlueprintBoundingBoxRapidBlueprint.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into IRapidBlueprint with oneOf schemas: IRapidBlueprintAttachCategoryRapidBlueprint, IRapidBlueprintBoundingBoxRapidBlueprint, IRapidBlueprintCompareRapidBlueprint, IRapidBlueprintFreeTextRapidBlueprint, IRapidBlueprintLineRapidBlueprint, IRapidBlueprintLocateRapidBlueprint, IRapidBlueprintNamedEntityRapidBlueprint, IRapidBlueprintPolygonRapidBlueprint, IRapidBlueprintScrubRapidBlueprint, IRapidBlueprintTranscriptionRapidBlueprint. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into IRapidBlueprint with oneOf schemas: IRapidBlueprintAttachCategoryRapidBlueprint, IRapidBlueprintBoundingBoxRapidBlueprint, IRapidBlueprintCompareRapidBlueprint, IRapidBlueprintFreeTextRapidBlueprint, IRapidBlueprintLineRapidBlueprint, IRapidBlueprintLocateRapidBlueprint, IRapidBlueprintNamedEntityRapidBlueprint, IRapidBlueprintPolygonRapidBlueprint, IRapidBlueprintScrubRapidBlueprint, IRapidBlueprintTranscriptionRapidBlueprint. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], IRapidBlueprintAttachCategoryRapidBlueprint, IRapidBlueprintBoundingBoxRapidBlueprint, IRapidBlueprintCompareRapidBlueprint, IRapidBlueprintFreeTextRapidBlueprint, IRapidBlueprintLineRapidBlueprint, IRapidBlueprintLocateRapidBlueprint, IRapidBlueprintNamedEntityRapidBlueprint, IRapidBlueprintPolygonRapidBlueprint, IRapidBlueprintScrubRapidBlueprint, IRapidBlueprintTranscriptionRapidBlueprint]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())





from __future__ import annotations

import subprocess
from importlib.metadata import version, PackageNotFoundError
from typing import TYPE_CHECKING

from rapidata.api_client.configuration import Configuration
from rapidata.service.credential_manager import CredentialManager
from rapidata.rapidata_client.api.rapidata_api_client import RapidataApiClient
from rapidata.rapidata_client.config import logger

if TYPE_CHECKING:
    from rapidata.api_client import CustomerRapidApi
    from rapidata.api_client.api.campaign_api import CampaignApi
    from rapidata.api_client.api.asset_api import AssetApi
    from rapidata.api_client.api.dataset_api import DatasetApi
    from rapidata.api_client.api.benchmark_api import BenchmarkApi
    from rapidata.api_client.api.order_api import OrderApi
    from rapidata.api_client.api.pipeline_api import PipelineApi
    from rapidata.api_client.api.leaderboard_api import LeaderboardApi
    from rapidata.api_client.api.validation_set_api import ValidationSetApi
    from rapidata.api_client.api.workflow_api import WorkflowApi
    from rapidata.api_client.api.participant_api import ParticipantApi


class OpenAPIService:
    def __init__(
        self,
        client_id: str | None,
        client_secret: str | None,
        environment: str,
        oauth_scope: str,
        cert_path: str | None = None,
        token: dict | None = None,
        leeway: int = 60,
    ):
        self.environment = environment
        endpoint = f"https://api.{environment}"
        auth_endpoint = f"https://auth.{environment}"

        if environment == "rapidata.dev" and not cert_path:
            cert_path = _get_local_certificate()

        logger.debug(
            "Using cert_path: %s environment: %s client_id: %s",
            cert_path,
            environment,
            client_id,
        )
        logger.debug("Initializing OpenAPIService")
        self.credential_manager = CredentialManager(
            endpoint=auth_endpoint, cert_path=cert_path
        )
        logger.debug("CredentialManager initialized")

        logger.debug("Initializing RapidataApiClient")
        client_configuration = Configuration(host=endpoint, ssl_ca_cert=cert_path)
        logger.debug("Client configuration: %s", client_configuration)
        self.api_client = RapidataApiClient(
            configuration=client_configuration,
            header_name="X-Client",
            header_value=f"RapidataPythonSDK/{self._get_rapidata_package_version()}",
        )
        logger.debug("RapidataApiClient initialized")

        if token:
            logger.debug("Using token for authentication")
            self.api_client.rest_client.setup_oauth_with_token(
                token=token,
                token_endpoint=f"{auth_endpoint}/connect/token",
                client_id=client_id,
                client_secret=client_secret,
                leeway=leeway,
            )
            logger.debug("Token authentication setup complete")
            return

        if not client_id or not client_secret:
            logger.debug(
                "Client ID and secret not provided, fetching from credential manager"
            )
            credentials = self.credential_manager.get_client_credentials()
            if not credentials:
                raise ValueError("Failed to fetch client credentials")
            client_id = credentials.client_id
            client_secret = credentials.client_secret

        self.api_client.rest_client.setup_oauth_client_credentials(
            client_id=client_id,
            client_secret=client_secret,
            token_endpoint=f"{auth_endpoint}/connect/token",
            scope=oauth_scope,
        )
        logger.debug("Client credentials authentication setup complete")

    def reset_credentials(self):
        self.credential_manager.reset_credentials()

    @property
    def order_api(self) -> OrderApi:
        from rapidata.api_client.api.order_api import OrderApi

        return OrderApi(self.api_client)

    @property
    def asset_api(self) -> AssetApi:
        from rapidata.api_client.api.asset_api import AssetApi

        return AssetApi(self.api_client)

    @property
    def dataset_api(self) -> DatasetApi:
        from rapidata.api_client.api.dataset_api import DatasetApi

        return DatasetApi(self.api_client)

    @property
    def validation_api(self) -> ValidationSetApi:
        from rapidata.api_client.api.validation_set_api import ValidationSetApi

        return ValidationSetApi(self.api_client)

    @property
    def rapid_api(self) -> CustomerRapidApi:
        from rapidata.api_client.api.customer_rapid_api import CustomerRapidApi

        return CustomerRapidApi(self.api_client)

    @property
    def campaign_api(self) -> CampaignApi:
        from rapidata.api_client.api.campaign_api import CampaignApi

        return CampaignApi(self.api_client)

    @property
    def pipeline_api(self) -> PipelineApi:
        from rapidata.api_client.api.pipeline_api import PipelineApi

        return PipelineApi(self.api_client)

    @property
    def workflow_api(self) -> WorkflowApi:
        from rapidata.api_client.api.workflow_api import WorkflowApi

        return WorkflowApi(self.api_client)

    @property
    def leaderboard_api(self) -> LeaderboardApi:
        from rapidata.api_client.api.leaderboard_api import LeaderboardApi

        return LeaderboardApi(self.api_client)

    @property
    def benchmark_api(self) -> BenchmarkApi:
        from rapidata.api_client.api.benchmark_api import BenchmarkApi

        return BenchmarkApi(self.api_client)

    @property
    def participant_api(self) -> ParticipantApi:
        from rapidata.api_client.api.participant_api import ParticipantApi

        return ParticipantApi(self.api_client)

    def _get_rapidata_package_version(self):
        """
        Returns the version of the currently installed rapidata package.

        Returns:
            Optional[str]: The version of the installed rapidata package,
                        or None if the package is not found
        """
        try:
            return version("rapidata")
        except PackageNotFoundError:
            return None

    def __repr__(self) -> str:
        return self.__str__()

    def __str__(self) -> str:
        return f"OpenAPIService(environment={self.environment})"


def _get_local_certificate() -> str | None:
    result = subprocess.run(["mkcert", "-CAROOT"], capture_output=True)
    if result.returncode != 0:
        return None

    output = result.stdout.decode("utf-8").strip()
    if not output:
        return None

    return f"{output}/rootCA.pem"



# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from rapidata.api_client.models.aggregator_type import AggregatorType
from rapidata.api_client.models.feature_flag_model import FeatureFlagModel
from rapidata.api_client.models.i_order_workflow_model import IOrderWorkflowModel
from rapidata.api_client.models.i_referee_model import IRefereeModel
from rapidata.api_client.models.i_selection import ISelection
from rapidata.api_client.models.i_user_filter_model import IUserFilterModel
from rapidata.api_client.models.retrieval_mode import RetrievalMode
from rapidata.api_client.models.sticky_state import StickyState
from typing import Optional, Set
from typing_extensions import Self

class CreateOrderModel(BaseModel):
    """
    CreateOrderModel
    """ # noqa: E501
    order_name: StrictStr = Field(alias="orderName")
    workflow: IOrderWorkflowModel
    referee: IRefereeModel
    aggregator: Optional[AggregatorType] = None
    feature_flags: Optional[List[FeatureFlagModel]] = Field(default=None, alias="featureFlags")
    priority: Optional[StrictInt] = None
    sticky_state: Optional[StickyState] = Field(default=None, alias="stickyState")
    user_score_dimensions: Optional[List[StrictStr]] = Field(default=None, alias="userScoreDimensions")
    demographic_keys: Optional[List[StrictStr]] = Field(default=None, alias="demographicKeys")
    user_filters: Optional[List[IUserFilterModel]] = Field(default=None, alias="userFilters")
    validation_set_id: Optional[StrictStr] = Field(default=None, alias="validationSetId")
    selections: Optional[List[ISelection]] = None
    retrieval_mode: Optional[RetrievalMode] = Field(default=None, alias="retrievalMode")
    max_iterations: Optional[StrictInt] = Field(default=None, alias="maxIterations")
    __properties: ClassVar[List[str]] = ["orderName", "workflow", "referee", "aggregator", "featureFlags", "priority", "stickyState", "userScoreDimensions", "demographicKeys", "userFilters", "validationSetId", "selections", "retrievalMode", "maxIterations"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateOrderModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of workflow
        if self.workflow:
            _dict['workflow'] = self.workflow.to_dict()
        # override the default output from pydantic by calling `to_dict()` of referee
        if self.referee:
            _dict['referee'] = self.referee.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in feature_flags (list)
        _items = []
        if self.feature_flags:
            for _item_feature_flags in self.feature_flags:
                if _item_feature_flags:
                    _items.append(_item_feature_flags.to_dict())
            _dict['featureFlags'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in user_filters (list)
        _items = []
        if self.user_filters:
            for _item_user_filters in self.user_filters:
                if _item_user_filters:
                    _items.append(_item_user_filters.to_dict())
            _dict['userFilters'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in selections (list)
        _items = []
        if self.selections:
            for _item_selections in self.selections:
                if _item_selections:
                    _items.append(_item_selections.to_dict())
            _dict['selections'] = _items
        # set to None if aggregator (nullable) is None
        # and model_fields_set contains the field
        if self.aggregator is None and "aggregator" in self.model_fields_set:
            _dict['aggregator'] = None

        # set to None if sticky_state (nullable) is None
        # and model_fields_set contains the field
        if self.sticky_state is None and "sticky_state" in self.model_fields_set:
            _dict['stickyState'] = None

        # set to None if validation_set_id (nullable) is None
        # and model_fields_set contains the field
        if self.validation_set_id is None and "validation_set_id" in self.model_fields_set:
            _dict['validationSetId'] = None

        # set to None if retrieval_mode (nullable) is None
        # and model_fields_set contains the field
        if self.retrieval_mode is None and "retrieval_mode" in self.model_fields_set:
            _dict['retrievalMode'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateOrderModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "orderName": obj.get("orderName"),
            "workflow": IOrderWorkflowModel.from_dict(obj["workflow"]) if obj.get("workflow") is not None else None,
            "referee": IRefereeModel.from_dict(obj["referee"]) if obj.get("referee") is not None else None,
            "aggregator": obj.get("aggregator"),
            "featureFlags": [FeatureFlagModel.from_dict(_item) for _item in obj["featureFlags"]] if obj.get("featureFlags") is not None else None,
            "priority": obj.get("priority"),
            "stickyState": obj.get("stickyState"),
            "userScoreDimensions": obj.get("userScoreDimensions"),
            "demographicKeys": obj.get("demographicKeys"),
            "userFilters": [IUserFilterModel.from_dict(_item) for _item in obj["userFilters"]] if obj.get("userFilters") is not None else None,
            "validationSetId": obj.get("validationSetId"),
            "selections": [ISelection.from_dict(_item) for _item in obj["selections"]] if obj.get("selections") is not None else None,
            "retrievalMode": obj.get("retrievalMode"),
            "maxIterations": obj.get("maxIterations")
        })
        return _obj





from rapidata.rapidata_client.settings._rapidata_setting import RapidataSetting


class Markdown(RapidataSetting):
    """
    Enable markdown rendering for text assets. Defaults to True.

    Args:
        value (bool): Whether to enable markdown rendering for text assets.
    """

    def __init__(self, value: bool = True):
        if not isinstance(value, bool):
            raise ValueError("The value must be a boolean.")
        super().__init__(key="use_text_asset_markdown", value=value)



import os
from typing import Literal

from rapidata.api_client.models.i_validation_truth_attach_category_truth import (
    IValidationTruthAttachCategoryTruth,
)
from rapidata.api_client.models.i_validation_truth_bounding_box_truth import (
    IValidationTruthBoundingBoxTruth,
)
from rapidata.api_client.models.i_rapid_payload_classify_payload import (
    IRapidPayloadClassifyPayload,
)
from rapidata.api_client.models.classify_payload_category import ClassifyPayloadCategory
from rapidata.api_client.models.i_rapid_payload_compare_payload import (
    IRapidPayloadComparePayload,
)
from rapidata.api_client.models.i_validation_truth_compare_truth import (
    IValidationTruthCompareTruth,
)
from rapidata.api_client.models.i_rapid_payload_line_payload import (
    IRapidPayloadLinePayload,
)
from rapidata.api_client.models.i_validation_truth_locate_box_truth import (
    IValidationTruthLocateBoxTruth,
)
from rapidata.api_client.models.i_rapid_payload_locate_payload import (
    IRapidPayloadLocatePayload,
)
from rapidata.api_client.models.i_rapid_payload_scrub_payload import (
    IRapidPayloadScrubPayload,
)
from rapidata.api_client.models.scrub_payload import ScrubPayload
from rapidata.api_client.models.scrub_range import ScrubRange
from rapidata.api_client.models.i_validation_truth_scrub_truth import (
    IValidationTruthScrubTruth,
)
from rapidata.api_client.models.scrub_truth import ScrubTruth
from rapidata.api_client.models.i_rapid_payload_transcription_payload import (
    IRapidPayloadTranscriptionPayload,
)
from rapidata.api_client.models.i_validation_truth_transcription_truth import (
    IValidationTruthTranscriptionTruth,
)
from rapidata.api_client.models.transcription_word import TranscriptionWord
from rapidata.rapidata_client.validation.rapids.box import Box
from rapidata.rapidata_client.validation.rapids.rapids import Rapid
from rapidata.service.openapi_service import OpenAPIService
from rapidata.api_client.models.i_rapid_payload import IRapidPayload


class RapidsManager:
    """
    Can be used to build different types of rapids. That can then be added to Validation sets
    """

    def __init__(self, openapi_service: OpenAPIService):
        self._openapi_service = openapi_service

    def classification_rapid(
        self,
        instruction: str,
        answer_options: list[str],
        datapoint: str,
        truths: list[str],
        data_type: Literal["media", "text"] = "media",
        context: str | None = None,
        media_context: str | None = None,
        explanation: str | None = None,
    ) -> Rapid:
        """Build a classification rapid

        Args:
            instruction (str): The instruction/question to be shown to the labeler.
            answer_options (list[str]): The options that the labeler can choose from to answer the question.
            datapoint (str): The datapoint that the labeler will be labeling.
            truths (list[str]): The correct answers to the question.
            data_type (str, optional): The type of the datapoint. Defaults to "media" (any form of image, video or audio).
            context (str, optional): The context is text that will be shown in addition to the instruction. Defaults to None.
            media_context (str, optional): The media context is a link to an image / video that will be shown in addition to the instruction (can be combined with context). Defaults to None.
            explanation (str, optional): The explanation that will be shown to the labeler if the answer is wrong. Defaults to None.
        """
        if not isinstance(truths, list):
            raise ValueError("Truths must be a list of strings")

        if not all(truth in answer_options for truth in truths):
            raise ValueError("Truths must be part of the answer options")

        payload = IRapidPayload(
            actual_instance=IRapidPayloadClassifyPayload(
                _t="ClassifyPayload",
                categories=[
                    ClassifyPayloadCategory(label=option, value=option)
                    for option in answer_options
                ],
                title=instruction,
            )
        )
        model_truth = IValidationTruthAttachCategoryTruth(
            correctCategories=truths, _t="AttachCategoryTruth"
        )

        return Rapid(
            asset=datapoint,
            data_type=data_type,
            context=context,
            media_context=media_context,
            explanation=explanation,
            payload=payload,
            truth=model_truth,
            random_correct_probability=len(truths) / len(answer_options),
        )

    def compare_rapid(
        self,
        instruction: str,
        truth: str,
        datapoint: list[str],
        data_type: Literal["media", "text"] = "media",
        context: str | None = None,
        media_context: str | None = None,
        explanation: str | None = None,
    ) -> Rapid:
        """Build a compare rapid

        Args:
            instruction (str): The instruction that the labeler will be comparing the assets on.
            truth (str): The correct answer to the comparison. (has to be one of the assets)
            datapoint (list[str]): The two assets that the labeler will be comparing.
            data_type (str, optional): The type of the datapoint. Defaults to "media" (any form of image, video or audio).
            context (str, optional): The context is text that will be shown in addition to the instruction. Defaults to None.
            media_context (str, optional): The media context is a link to an image / video that will be shown in addition to the instruction (can be combined with context). Defaults to None.
            explanation (str, optional): The explanation that will be shown to the labeler if the answer is wrong. Defaults to None.
        """

        payload = IRapidPayload(
            actual_instance=IRapidPayloadComparePayload(
                _t="ComparePayload", criteria=instruction
            )
        )
        truth = os.path.basename(truth)
        model_truth = IValidationTruthCompareTruth(_t="CompareTruth", winnerId=truth)

        if len(datapoint) != 2:
            raise ValueError("Compare rapid requires exactly two media paths")

        return Rapid(
            asset=datapoint,
            data_type=data_type,
            truth=model_truth,
            context=context,
            media_context=media_context,
            payload=payload,
            explanation=explanation,
            random_correct_probability=0.5,
        )

    def select_words_rapid(
        self,
        instruction: str,
        truths: list[int],
        datapoint: str,
        sentence: str,
        required_precision: float = 1,
        required_completeness: float = 1,
        explanation: str | None = None,
    ) -> Rapid:
        """Build a select words rapid

        Args:
            instruction (str): The instruction for the labeler.
            truths (list[int]): The indices of the words that are the correct answers.
            datapoint (str): The asset that the labeler will be selecting words from.
            sentence (str): The sentence that the labeler will be selecting words from. (split up by spaces)
            required_precision (float): The required precision for the labeler to get the rapid correct (minimum ratio of the words selected that need to be correct). defaults to 1. (no wrong words can be selected)
            required_completeness (float): The required completeness for the labeler to get the rapid correct (miminum ratio of total correct words selected). defaults to 1. (all correct words need to be selected)
            explanation (str, optional): The explanation that will be shown to the labeler if the answer is wrong. Defaults to None.
        """

        transcription_words = [
            TranscriptionWord(word=word, wordIndex=i)
            for i, word in enumerate(sentence.split(" "))
        ]

        correct_transcription_words: list[TranscriptionWord] = []
        for index in truths:
            correct_transcription_words.append(
                TranscriptionWord(word=transcription_words[index].word, wordIndex=index)
            )

        payload = IRapidPayload(
            actual_instance=IRapidPayloadTranscriptionPayload(
                _t="TranscriptionPayload",
                title=instruction,
                transcription=transcription_words,
            )
        )

        model_truth = IValidationTruthTranscriptionTruth(
            _t="TranscriptionTruth",
            correctWords=correct_transcription_words,
            requiredPrecision=required_precision,
            requiredCompleteness=required_completeness,
        )

        return Rapid(
            payload=payload,
            truth=model_truth,
            asset=datapoint,
            sentence=sentence,
            explanation=explanation,
            random_correct_probability=len(correct_transcription_words)
            / len(transcription_words),
        )

    def locate_rapid(
        self,
        instruction: str,
        truths: list[Box],
        datapoint: str,
        context: str | None = None,
        media_context: str | None = None,
        explanation: str | None = None,
    ) -> Rapid:
        """Build a locate rapid

        Args:
            instruction (str): The instruction on what the labeler should do.
            truths (list[Box]): The bounding boxes of the object that the labeler ought to be locating.
            datapoint (str): The asset that the labeler will be locating the object in.
            context (str, optional): The context is text that will be shown in addition to the instruction. Defaults to None.
            media_context (str, optional): The media context is a link to an image / video that will be shown in addition to the instruction (can be combined with context). Defaults to None.
            explanation (str, optional): The explanation that will be shown to the labeler if the answer is wrong. Defaults to None.
        """

        payload = IRapidPayload(
            actual_instance=IRapidPayloadLocatePayload(
                _t="LocatePayload", target=instruction
            )
        )

        model_truth = IValidationTruthLocateBoxTruth(
            _t="LocateBoxTruth",
            boundingBoxes=[truth.to_model() for truth in truths],
        )

        coverage = self._calculate_boxes_coverage(
            truths,
        )

        return Rapid(
            payload=payload,
            truth=model_truth,
            asset=datapoint,
            context=context,
            media_context=media_context,
            explanation=explanation,
            random_correct_probability=coverage,
        )

    def draw_rapid(
        self,
        instruction: str,
        truths: list[Box],
        datapoint: str,
        context: str | None = None,
        media_context: str | None = None,
        explanation: str | None = None,
    ) -> Rapid:
        """Build a draw rapid

        Args:
            instruction (str): The instructions on what the labeler
            truths (list[Box]): The bounding boxes of the object that the labeler ought to be drawing.
            datapoint (str): The asset that the labeler will be drawing the object in.
            context (str, optional): The context is text that will be shown in addition to the instruction. Defaults to None.
            media_context (str, optional): The media context is a link to an image / video that will be shown in addition to the instruction (can be combined with context). Defaults to None.
            explanation (str, optional): The explanation that will be shown to the labeler if the answer is wrong. Defaults to None.
        """

        payload = IRapidPayload(
            actual_instance=IRapidPayloadLinePayload(
                _t="LinePayload", target=instruction
            )
        )

        model_truth = IValidationTruthBoundingBoxTruth(
            _t="BoundingBoxTruth",
            xMax=truths[0].x_max * 100,
            xMin=truths[0].x_min * 100,
            yMax=truths[0].y_max * 100,
            yMin=truths[0].y_min * 100,
        )

        coverage = self._calculate_boxes_coverage(
            truths,
        )

        return Rapid(
            payload=payload,
            truth=model_truth,
            asset=datapoint,
            context=context,
            media_context=media_context,
            explanation=explanation,
            random_correct_probability=coverage,
        )

    def timestamp_rapid(
        self,
        instruction: str,
        truths: list[tuple[int, int]],
        datapoint: str,
        context: str | None = None,
        media_context: str | None = None,
        explanation: str | None = None,
    ) -> Rapid:
        """Build a timestamp rapid

        Args:
            instruction (str): The instruction for the labeler.
            truths (list[tuple[int, int]]): The possible accepted timestamps intervals for the labeler (in miliseconds).
                The first element of the tuple is the start of the interval and the second element is the end of the interval.
            datapoint (str): The asset that the labeler will be timestamping.
            context (str, optional): The context is text that will be shown in addition to the instruction. Defaults to None.
            media_context (str, optional): The media context is a link to an image / video that will be shown in addition to the instruction (can be combined with context). Defaults to None.
            explanation (str, optional): The explanation that will be shown to the labeler if the answer is wrong. Defaults to None.
        """

        for truth in truths:
            if len(truth) != 2:
                raise ValueError(
                    "The truths per datapoint must be a tuple of exactly two integers."
                )
            if truth[0] > truth[1]:
                raise ValueError(
                    "The start of the interval must be smaller than the end of the interval."
                )

        payload = IRapidPayload(
            actual_instance=IRapidPayloadScrubPayload(
                _t="ScrubPayload", target=instruction
            )
        )

        model_truth = IValidationTruthScrubTruth(
            _t="ScrubTruth",
            validRanges=[ScrubRange(start=truth[0], end=truth[1]) for truth in truths],
        )

        return Rapid(
            payload=payload,
            truth=model_truth,
            asset=datapoint,
            context=context,
            media_context=media_context,
            explanation=explanation,
            random_correct_probability=0.5,  # TODO: implement coverage ratio
        )

    def _calculate_boxes_coverage(self, boxes: list[Box]) -> float:
        """
        Calculate the ratio of area covered by a list of boxes.

        Args:
            boxes: List of Box objects with coordinates in range [0, 1]

        Returns:
            float: Coverage ratio between 0.0 and 1.0
        """
        if not boxes:
            return 0.0

        # Convert boxes to intervals for sweep line algorithm
        events = []

        # Create events for x-coordinates
        for i, box in enumerate(boxes):
            events.append((box.x_min, "start", i, box))
            events.append((box.x_max, "end", i, box))

        # Sort events by x-coordinate
        events.sort(key=lambda x: (x[0], x[1] == "end"))

        total_area = 0.0
        active_boxes = set()
        prev_x = 0.0

        for x, event_type, box_id, box in events:
            # Calculate area for the previous x-interval
            if active_boxes and x > prev_x:
                # Merge y-intervals for active boxes
                y_intervals = [(boxes[i].y_min, boxes[i].y_max) for i in active_boxes]
                y_intervals.sort()

                # Merge overlapping y-intervals
                merged_intervals = []
                for start, end in y_intervals:
                    if merged_intervals and start <= merged_intervals[-1][1]:
                        # Overlapping intervals - merge them
                        merged_intervals[-1] = (
                            merged_intervals[-1][0],
                            max(merged_intervals[-1][1], end),
                        )
                    else:
                        # Non-overlapping interval
                        merged_intervals.append((start, end))

                # Calculate total y-coverage for this x-interval
                y_coverage = sum(end - start for start, end in merged_intervals)
                total_area += (x - prev_x) * y_coverage

            # Update active boxes
            if event_type == "start":
                active_boxes.add(box_id)
            else:
                active_boxes.discard(box_id)

            prev_x = x

        return total_area

    @staticmethod
    def _calculate_coverage_ratio(
        total_duration: int, subsections: list[tuple[int, int]]
    ) -> float:
        """
        Calculate the ratio of total_duration that is covered by subsections, handling overlaps.

        Args:
            total_duration: The total duration to consider
            subsections: List of tuples containing (start, end) times

        Returns:
            float: Ratio of coverage (0 to 1)
        """
        if not subsections:
            return 0.0

        # Sort subsections by start time and clamp to valid range
        sorted_ranges = sorted(
            (max(0, start), min(end, total_duration)) for start, end in subsections
        )

        # Merge overlapping ranges
        merged_ranges = []
        current_range = list(sorted_ranges[0])

        for next_start, next_end in sorted_ranges[1:]:
            current_start, current_end = current_range

            # If ranges overlap or are adjacent
            if next_start <= current_end:
                current_range[1] = max(current_end, next_end)
            else:
                merged_ranges.append(current_range)
                current_range = [next_start, next_end]

        merged_ranges.append(current_range)

        # Calculate total coverage
        total_coverage = sum(end - start for start, end in merged_ranges)

        return total_coverage / total_duration

    def __str__(self) -> str:
        return "RapidsManager"

    def __repr__(self) -> str:
        return self.__str__()



from pydantic import BaseModel, Field

from rapidata.rapidata_client.config.logging_config import LoggingConfig
from rapidata.rapidata_client.config.order_config import OrderConfig
from rapidata.rapidata_client.config.upload_config import UploadConfig


class RapidataConfig(BaseModel):
    """
    Holds the configuration for the Rapidata client.

    To adjust the configurations used, you can modify the `rapidata_config` object.

    Attributes:
        enableBetaFeatures (bool): Whether to enable beta features. Defaults to False.
        upload (UploadConfig): The configuration for the upload process.
            Such as the maximum number of worker threads for processing media paths and the maximum number of retries for failed uploads.
        order (OrderConfig): The configuration for the order process.
            Such as the minimum number of datapoints required so that an automatic validationset gets created if no recommended was found.
        logging (LoggingConfig): The configuration for the logging process.
            Such as the logging level and the logging file.

    Example:
        ```python
        from rapidata import rapidata_config
        rapidata_config.upload.maxUploadWorkers = 20
        ```
    """

    enableBetaFeatures: bool = False
    upload: UploadConfig = Field(default_factory=UploadConfig)
    order: OrderConfig = Field(default_factory=OrderConfig)
    logging: LoggingConfig = Field(default_factory=LoggingConfig)


rapidata_config = RapidataConfig()



from rapidata.api_client.models.private_text_metadata_input import (
    PrivateTextMetadataInput,
)
from rapidata.rapidata_client.datapoints.metadata._base_metadata import Metadata
from pydantic import BaseModel


class PrivateTextMetadata(Metadata, BaseModel):

    text: str
    identifier: str = "private_text"

    def to_model(self):
        return PrivateTextMetadataInput(
            _t="PrivateTextMetadataInput", identifier=self.identifier, text=self.text
        )



from typing import Optional, cast
from rapidata.rapidata_client.benchmark.rapidata_benchmark import RapidataBenchmark
from rapidata.api_client.models.create_benchmark_model import CreateBenchmarkModel
from rapidata.service.openapi_service import OpenAPIService
from rapidata.api_client.models.query_model import QueryModel
from rapidata.api_client.models.page_info import PageInfo
from rapidata.api_client.models.root_filter import RootFilter
from rapidata.api_client.models.filter import Filter
from rapidata.api_client.models.sort_criterion import SortCriterion
from rapidata.api_client.models.sort_direction import SortDirection
from rapidata.api_client.models.filter_operator import FilterOperator
from rapidata.rapidata_client.config import logger, tracer


class RapidataBenchmarkManager:
    """
    A manager for benchmarks.

    Used to create and retrieve benchmarks.

    A benchmark is a collection of leaderboards.
    """

    def __init__(self, openapi_service: OpenAPIService):
        self.__openapi_service = openapi_service

    def create_new_benchmark(
        self,
        name: str,
        identifiers: Optional[list[str]] = None,
        prompts: Optional[list[str | None] | list[str]] = None,
        prompt_assets: Optional[list[str | None] | list[str]] = None,
        tags: Optional[list[list[str] | None] | list[list[str]]] = None,
    ) -> RapidataBenchmark:
        """
        Creates a new benchmark with the given name, identifiers, prompts, and media assets.
        Everything is matched up by the indexes of the lists.

        prompts or identifiers must be provided, as well as prompts or prompt_assets.

        Args:
            name: The name of the benchmark.
            identifiers: The identifiers of the prompts/assets/tags that will be used to match up the media. If not provided, it will use the prompts as the identifiers.
            prompts: The prompts that will be registered for the benchmark.
            prompt_assets: The prompt assets that will be registered for the benchmark.
            tags: The tags that will be associated with the prompts to use for filtering the leaderboard results. They will NOT be shown to the users.

        Example:
            ```python
            name = "Example Benchmark"
            identifiers = ["id1", "id2", "id3"]
            prompts = ["prompt 1", "prompt 2", "prompt 3"]
            prompt_assets = ["https://assets.rapidata.ai/prompt_1.jpg", "https://assets.rapidata.ai/prompt_2.jpg", "https://assets.rapidata.ai/prompt_3.jpg"]
            tags = [["tag1", "tag2"], ["tag2"], ["tag2", "tag3"]]

            benchmark = create_new_benchmark(name=name, identifiers=identifiers, prompts=prompts, prompt_assets=prompt_assets, tags=tags)
            ```
        """
        with tracer.start_as_current_span(
            "RapidataBenchmarkManager.create_new_benchmark"
        ):
            if not isinstance(name, str):
                raise ValueError("Name must be a string.")

            if prompts and (
                not isinstance(prompts, list)
                or not all(
                    isinstance(prompt, str) or prompt is None for prompt in prompts
                )
            ):
                raise ValueError("Prompts must be a list of strings or None.")

            if prompt_assets and (
                not isinstance(prompt_assets, list)
                or not all(
                    isinstance(asset, str) or asset is None for asset in prompt_assets
                )
            ):
                raise ValueError("Media assets must be a list of strings or None.")

            if identifiers and (
                not isinstance(identifiers, list)
                or not all(isinstance(identifier, str) for identifier in identifiers)
            ):
                raise ValueError("Identifiers must be a list of strings.")

            if identifiers:
                if not len(set(identifiers)) == len(identifiers):
                    raise ValueError("Identifiers must be unique.")

            if tags is not None:
                if not isinstance(tags, list):
                    raise ValueError("Tags must be a list of lists of strings or None.")

                for tag in tags:
                    if tag is not None and (
                        not isinstance(tag, list)
                        or not all(isinstance(item, str) for item in tag)
                    ):
                        raise ValueError(
                            "Tags must be a list of lists of strings or None."
                        )

            if not identifiers and not prompts:
                raise ValueError(
                    "At least one of identifiers or prompts must be provided."
                )

            if not prompts and not prompt_assets:
                raise ValueError(
                    "At least one of prompts or media assets must be provided."
                )

            if not identifiers:
                assert prompts is not None
                if not len(set(prompts)) == len(prompts):
                    raise ValueError(
                        "Prompts must be unique. Otherwise use identifiers."
                    )
                if any(prompt is None for prompt in prompts):
                    raise ValueError(
                        "Prompts must not be None. Otherwise use identifiers."
                    )

                identifiers = cast(list[str], prompts)

            assert identifiers is not None

            expected_length = len(identifiers)

            if not prompts:
                prompts = cast(list[str | None], [None] * expected_length)

            if not prompt_assets:
                prompt_assets = cast(list[str | None], [None] * expected_length)

            if not tags:
                tags = cast(list[list[str] | None], [None] * expected_length)

            # At this point, all variables are guaranteed to be lists, not None
            assert prompts is not None
            assert prompt_assets is not None
            assert tags is not None

            if not (expected_length == len(prompts) == len(prompt_assets) == len(tags)):
                raise ValueError(
                    "Identifiers, prompts, media assets, and tags must have the same length or set to None."
                )

            logger.info("Creating new benchmark %s", name)

            benchmark_result = self.__openapi_service.benchmark_api.benchmark_post(
                create_benchmark_model=CreateBenchmarkModel(
                    name=name,
                )
            )

            logger.info("Benchmark created with id %s", benchmark_result.id)

            benchmark = RapidataBenchmark(
                name, benchmark_result.id, self.__openapi_service
            )

            for identifier, prompt, asset, tag in zip(
                identifiers, prompts, prompt_assets, tags
            ):
                benchmark.add_prompt(identifier, prompt, asset, tag)

            return benchmark

    def get_benchmark_by_id(self, id: str) -> RapidataBenchmark:
        """
        Returns a benchmark by its ID.
        """
        with tracer.start_as_current_span(
            "RapidataBenchmarkManager.get_benchmark_by_id"
        ):
            benchmark_result = (
                self.__openapi_service.benchmark_api.benchmark_benchmark_id_get(
                    benchmark_id=id
                )
            )
            return RapidataBenchmark(
                benchmark_result.name, benchmark_result.id, self.__openapi_service
            )

    def find_benchmarks(
        self, name: str = "", amount: int = 10
    ) -> list[RapidataBenchmark]:
        """
        Returns a list of benchmarks by their name.
        """
        with tracer.start_as_current_span("RapidataBenchmarkManager.find_benchmarks"):
            benchmark_result = self.__openapi_service.benchmark_api.benchmarks_get(
                QueryModel(
                    page=PageInfo(index=1, size=amount),
                    filter=RootFilter(
                        filters=[
                            Filter(
                                field="Name",
                                operator=FilterOperator.CONTAINS,
                                value=name,
                            )
                        ]
                    ),
                    sortCriteria=[
                        SortCriterion(
                            direction=SortDirection.DESC, propertyName="CreatedAt"
                        )
                    ],
                )
            )
            return [
                RapidataBenchmark(benchmark.name, benchmark.id, self.__openapi_service)
                for benchmark in benchmark_result.items
            ]



from __future__ import annotations

from rapidata.rapidata_client.filter._base_filter import RapidataFilter
from pydantic import BaseModel, ConfigDict


class NotFilter(RapidataFilter, BaseModel):
    """A filter that negates another filter's condition.
    This class implements a logical NOT operation on a given filter, inverting its results.

    Args:
        filter (RapidataFilter): The filter whose condition should be negated.

    Example:
        ```python
        from rapidata import NotFilter, LanguageFilter

        NotFilter(LanguageFilter(["en"]))
        ```

        This will limit the order to be shown to only people who have their phone set to a language other than English.
    """

    model_config = ConfigDict(arbitrary_types_allowed=True)

    filter: RapidataFilter

    def __init__(self, filter: RapidataFilter):
        super().__init__(filter=filter)

    def _to_model(self):
        from rapidata.api_client.models.i_user_filter_model import IUserFilterModel
        from rapidata.api_client.models.i_user_filter_model_not_user_filter_model import (
            IUserFilterModelNotUserFilterModel,
        )

        return IUserFilterModel(
            actual_instance=IUserFilterModelNotUserFilterModel(
                _t="NotFilter",
                filter=self.filter._to_model(),
            )
        )



from __future__ import annotations

from typing import Sequence

from rapidata.rapidata_client.selection._base_selection import RapidataSelection


class AbTestSelection(RapidataSelection):
    """AbTestSelection Class

    Splits the userbase into two segments and serves them a different collection of rapids.

    Useful for A/B Test.

    Args:
        a_selections (Sequence[RapidataSelection]): List of selections for group A.
        b_selections (Sequence[RapidataSelection]): List of selections for group B.
    """

    def __init__(
        self,
        a_selections: Sequence[RapidataSelection],
        b_selections: Sequence[RapidataSelection],
    ):
        self.a_selections = a_selections
        self.b_selections = b_selections

    def _to_model(self):
        from rapidata.api_client.models.i_selection import ISelection
        from rapidata.api_client.models.i_selection_ab_test_selection import (
            ISelectionAbTestSelection,
        )

        return ISelection(
            actual_instance=ISelectionAbTestSelection(
                _t="AbTestSelection",
                a=[selection._to_model() for selection in self.a_selections],
                b=[selection._to_model() for selection in self.b_selections],
            )
        )

    def __str__(self) -> str:
        return f"AbTestSelection(a_selections={self.a_selections}, b_selections={self.b_selections})"

    def __repr__(self) -> str:
        return self.__str__()



from __future__ import annotations

from rapidata.rapidata_client.filter._base_filter import RapidataFilter
from pydantic import BaseModel, field_validator
from rapidata.rapidata_client.config import logger


class CountryFilter(RapidataFilter, BaseModel):
    """CountryFilter Class

    Can be used to filter who to target based on country codes.

    Args:
        country_codes (list[str]): List of country codes (capitalized) to filter by.
    """

    country_codes: list[str]

    def __init__(self, country_codes: list[str]):
        super().__init__(country_codes=country_codes)

    @field_validator("country_codes")
    @classmethod
    def validate_country_codes(cls, codes: list[str]) -> list[str]:
        validated = []
        for code in codes:
            if len(code) != 2:
                raise ValueError(
                    f"Country codes must be length 2. Invalid code: '{code}'"
                )
            if code != code.upper():
                logger.warning(
                    f"Country code '{code}' should be uppercase. It will be uppercased automatically."
                )
            validated.append(code.upper())
        return validated

    def _to_model(self):
        from rapidata.api_client.models.i_user_filter_model import IUserFilterModel
        from rapidata.api_client.models.i_user_filter_model_country_user_filter_model import (
            IUserFilterModelCountryUserFilterModel,
        )

        return IUserFilterModel(
            actual_instance=IUserFilterModelCountryUserFilterModel(
                _t="CountryFilter", countries=self.country_codes
            )
        )
